from PySide6.QtCore import Qt
from PySide6.QtGui import QColor, QRgba64
from typing import TypeAlias, Union, Any
from enum import Enum


class Colors(Enum):
    _49ers_red: QColor = QColor("#AA0000")
    abalone: QColor = QColor("#D6CFC7")
    abbey: QColor = QColor("#494F55")
    abbey_white: QColor = QColor("#EBE5D0")
    abstract_white: QColor = QColor("#EDE9DD")
    acadia_white: QColor = QColor("#F4EFE3")
    acid_green: QColor = QColor("#B0BF1A")
    acorn: QColor = QColor("#D7A98C")
    acoustic_white: QColor = QColor("#EFECE1")
    admiral: QColor = QColor("#051094")
    admiral_blue: QColor = QColor("#2C3863")
    aegean: QColor = QColor("#1F456E")
    aegean_blue: QColor = QColor("#4E6E81")
    aero_blue: QColor = QColor("#C9FFE5")
    aesthetic_black: QColor = QColor("#1C1C1E")
    aesthetic_orange: QColor = QColor("#F4AB6A")
    aesthetic_purple: QColor = QColor("#502380")
    aesthetic_yellow: QColor = QColor("#EFD033")
    affair: QColor = QColor("#6F4685")
    african_violet: QColor = QColor("#B284BE")
    agreeable_gray: QColor = QColor("#D1CBC1")
    air_force_blue_raf: QColor = QColor("#5D8AA8")
    air_force_blue_usaf: QColor = QColor("#00308F")
    air_superiority_blue: QColor = QColor("#72A0C1")
    airbnb_red: QColor = QColor("#FF5A5F")
    aircraft_white: QColor = QColor("#EDF2F8")
    ajax_red: QColor = QColor("#D2122E")
    alabama_crimson: QColor = QColor("#9E1B32")
    alabaster: QColor = QColor("#EDEAE0")
    alaskan_blue: QColor = QColor("#6DA9D2")
    albescent_white: QColor = QColor("#F3DECA")
    alice_blue: QColor = QColor("#F0F8FF")
    alien_black: QColor = QColor("#1A2228")
    alizarin_crimson: QColor = QColor("#E32636")
    alloy_orange: QColor = QColor("#C35214")
    almond: QColor = QColor("#EFDECD")
    almost_apricot_red: QColor = QColor("#E5B39B")
    amaranth: QColor = QColor("#E52B50")
    amaranth_pink: QColor = QColor("#E52B50")
    amaranth_purple: QColor = QColor("#AB274F")
    amaranth_red: QColor = QColor("#F4364C")
    amazon: QColor = QColor("#3B7A57")
    amber: QColor = QColor("#FFBF00")
    american_rose: QColor = QColor("#FF033E")
    amethyst: QColor = QColor("#9966CC")
    anchor: QColor = QColor("#3E424B")
    android_green: QColor = QColor("#A4C639")
    angels_red: QColor = QColor("#BA0021")
    anti_flash_white: QColor = QColor("#F2F3F4")
    antique_brass: QColor = QColor("#CD9575")
    antique_pewter: QColor = QColor("#808992")
    antique_white: QColor = QColor("#FAEBD7")
    antique_white_usa: QColor = QColor("#EEEAE1")
    anvil_gray: QColor = QColor("#6B6B6B")
    apple: QColor = QColor("#A91B0D")
    apple_green: QColor = QColor("#8DB600")
    apricot: QColor = QColor("#ED820E")
    apricot_orange: QColor = QColor("#FBCEB1")
    aqua_blue: QColor = QColor("#0AFFFF")
    aquamarine_blue: QColor = QColor("#6BCAE2")
    arctic: QColor = QColor("#82EEFD")
    arctic_blue: QColor = QColor("#C6E6FB")
    arctic_lime: QColor = QColor("#D0FF14")
    argentina_blue: QColor = QColor("#6CB4EE")
    army_green: QColor = QColor("#4B5320")
    army_uniform: QColor = QColor("#353F3E")
    arsenal_red: QColor = QColor("#DB0007")
    arsenic: QColor = QColor("#3B444B")
    artichoke_green: QColor = QColor("#8F9779")
    artists_purple: QColor = QColor("#C71585")
    arylide_yellow: QColor = QColor("#E9D66B")
    ash: QColor = QColor("#564C4D")
    ash_brown: QColor = QColor("#98623C")
    ash_gray: QColor = QColor("#B2BEB5")
    ash_white: QColor = QColor("#E9E4D4")
    asparagus: QColor = QColor("#87A96B")
    asphalt: QColor = QColor("#0C0404")
    astros_navy: QColor = QColor("#002D62")
    atomic_tangerine: QColor = QColor("#FF9966")
    atrium_white: QColor = QColor("#F1EEE4")
    auburn: QColor = QColor("#A52A2A")
    auburn_red: QColor = QColor("#A52A2A")
    audience_anger: QColor = QColor("#EAF4FC")
    aureolin: QColor = QColor("#FDEE00")
    autumn_purple: QColor = QColor("#834468")
    avocado: QColor = QColor("#568203")
    aztech_purple: QColor = QColor("#893BFF")
    azure: QColor = QColor("#F0FFFF")
    azureish_white: QColor = QColor("#DBE9F4")
    baby_blue: QColor = QColor("#89CFF0")
    baby_blue_eyes: QColor = QColor("#A1CAF1")
    baby_orange: QColor = QColor("#FCBE85")
    baby_pink: QColor = QColor("#F4C2C2")
    baby_powder: QColor = QColor("#FEFEFA")
    baby_yellow: QColor = QColor("#FCEF91")
    baked_bread: QColor = QColor("#DACBA9")
    baked_potato: QColor = QColor("#B69E87")
    ballet_slipper: QColor = QColor("#F79AC0")
    baltimore_ravens_purple: QColor = QColor("#280353")
    bamboo: QColor = QColor("#E3DEC6")
    banana: QColor = QColor("#FFE135")
    barbie_pink: QColor = QColor("#E0218A")
    bark: QColor = QColor("#996633")
    barn_red: QColor = QColor("#7C0A02")
    basalt_black: QColor = QColor("#4D423E")
    basil: QColor = QColor("#32612D")
    battleship_gray: QColor = QColor("#848482")
    bayern_blue: QColor = QColor("#0066B2")
    bdazzled: QColor = QColor("#2E5894")
    bdazzled_blue: QColor = QColor("#2E5894")
    beau_blue: QColor = QColor("#BCD4E6")
    beaver: QColor = QColor("#9F8170")
    beer_orange: QColor = QColor("#F28F1C")
    beeswax: QColor = QColor("#E9D7AB")
    beige: QColor = QColor("#F5F5DC")
    beige_linen: QColor = QColor("#E2DAC6")
    berry: QColor = QColor("#7A1712")
    berry_blue: QColor = QColor("#4F86F7")
    best_gray: QColor = QColor("#969892")
    bills_red: QColor = QColor("#C60C30")
    birdseed: QColor = QColor("#E2C28E")
    biscotti: QColor = QColor("#E3C565")
    bistre: QColor = QColor("#3D2B1F")
    bitter_chocolate: QColor = QColor("#9E5B40")
    bitter_lemon: QColor = QColor("#CAE00D")
    bitter_lime: QColor = QColor("#BFFF00")
    bittersweet: QColor = QColor("#FE6F5E")
    bittersweet_shimmer: QColor = QColor("#BF4F51")
    bittersweet_shimmer_red: QColor = QColor("#BF4F51")
    black: QColor = QColor("#000000")
    black_bean: QColor = QColor("#3D0C02")
    black_blue_ral: QColor = QColor("#1A1E28")
    black_cat: QColor = QColor("#413839")
    black_chocolate: QColor = QColor("#1B1811")
    black_coral: QColor = QColor("#54626F")
    black_cow: QColor = QColor("#4C4646")
    black_currant: QColor = QColor("#540E32")
    black_denim: QColor = QColor("#191C27")
    black_eel: QColor = QColor("#463E3F")
    black_grain: QColor = QColor("#2C2C2A")
    black_gray: QColor = QColor("#303234")
    black_green_ral: QColor = QColor("#303D3A")
    black_hole: QColor = QColor("#060605")
    black_leather_jacket: QColor = QColor("#253529")
    black_magic: QColor = QColor("#0B0510")
    black_olive: QColor = QColor("#3B3C36")
    black_pearl: QColor = QColor("#0E161A")
    black_plum: QColor = QColor("#362B32")
    black_raspberry: QColor = QColor("#451425")
    black_red: QColor = QColor("#3D2022")
    black_rock: QColor = QColor("#010127")
    black_russian: QColor = QColor("#24252B")
    black_shadows: QColor = QColor("#BFAFB2")
    blackberry: QColor = QColor("#3A3A38")
    blackish_purple: QColor = QColor("#291E29")
    blackstrap_molasses: QColor = QColor("#2B2523")
    blanched_almond: QColor = QColor("#FFEBCD")
    blazing_yellow: QColor = QColor("#FEE715")
    bleu_de_france: QColor = QColor("#318CE7")
    bleu_de_france_blue: QColor = QColor("#318CE7")
    blood: QColor = QColor("#710C04")
    blood_red: QColor = QColor("#660000")
    blue: QColor = QColor("#0000FF")
    blue_charcoal: QColor = QColor("#262B2F")
    blue_gray: QColor = QColor("#98AFC7")
    blue_lilac: QColor = QColor("#7B679A")
    blue_sapphire: QColor = QColor("#126180")
    blue_violet: QColor = QColor("#8A2BE2")
    blue_yonder: QColor = QColor("#5072A7")
    blueberry: QColor = QColor("#241571")
    blush: QColor = QColor("#DE5D83")
    blush_pink: QColor = QColor("#FEC5E5")
    blush_red: QColor = QColor("#BC544B")
    boeing_blue: QColor = QColor("#0039A6")
    bole: QColor = QColor("#79443B")
    bone: QColor = QColor("#E3DAC9")
    bosss_jokes: QColor = QColor("#B0306A")
    bottle_green: QColor = QColor("#006A4E")
    boysenberry: QColor = QColor("#630436")
    brandy: QColor = QColor("#DCB68A")
    brass: QColor = QColor("#B5A642")
    braves_navy: QColor = QColor("#13274F")
    brewers_blue: QColor = QColor("#0A2351")
    brick: QColor = QColor("#7E2811")
    bright_black: QColor = QColor("#222024")
    bright_canary_yellow: QColor = QColor("#FFE41E")
    bright_gray: QColor = QColor("#E8E9EB")
    bright_maroon: QColor = QColor("#C32148")
    bright_mint: QColor = QColor("#4FFFB0")
    bright_pink: QColor = QColor("#FF007F")
    bright_turquoise: QColor = QColor("#08E8DE")
    bright_white: QColor = QColor("#F4F5F0")
    bright_yellow: QColor = QColor("#FFFD01")
    brilliant_purple: QColor = QColor("#D399E6")
    brilliant_rose: QColor = QColor("#FF55A3")
    brilliant_white: QColor = QColor("#EDF1FE")
    brink: QColor = QColor("#FF6090")
    british_racing_green: QColor = QColor("#004225")
    british_standard_bs_4800_10_e_51_yellow: QColor = QColor("#EDC001")
    broken_white: QColor = QColor("#EEEBE3")
    broncos_blue: QColor = QColor("#002244")
    bronze: QColor = QColor("#B2560D")
    bronze_orange: QColor = QColor("#CD7F32")
    bronze_yellow: QColor = QColor("#737000")
    broom_yellow: QColor = QColor("#DBA400")
    brown_bear: QColor = QColor("#7F6244")
    brown_beige: QColor = QColor("#CC8833")
    brown_coffee: QColor = QColor("#4A2C2A")
    brown_gray: QColor = QColor("#8D8468")
    brown_metal: QColor = QColor("#BBADA1")
    brown_oil: QColor = QColor("#231709")
    brown_red: QColor = QColor("#922B05")
    brown_rust: QColor = QColor("#AF593E")
    brown_stone: QColor = QColor("#593C39")
    brown_sugar: QColor = QColor("#A17249")
    brownie: QColor = QColor("#964B00")
    brownish_gray: QColor = QColor("#86775F")
    brownish_red: QColor = QColor("#9E3623")
    browns_orange: QColor = QColor("#FF3C00")
    brunette: QColor = QColor("#3B1E08")
    bu_scarlet: QColor = QColor("#CC0000")
    bubblegum: QColor = QColor("#FFC1CC")
    bubbles: QColor = QColor("#E7FEFF")
    buff_orange: QColor = QColor("#F2CA85")
    buff_yellow: QColor = QColor("#F1BF70")
    bumblebee: QColor = QColor("#FCE205")
    bumblebee_orange: QColor = QColor("#FFC82A")
    burgundy: QColor = QColor("#8D021F")
    burlywood: QColor = QColor("#DEB887")
    burnt_almond: QColor = QColor("#B0724A")
    burnt_henna: QColor = QColor("#7E392F")
    burnt_orange: QColor = QColor("#CC5500")
    burnt_sienna: QColor = QColor("#E97451")
    burnt_umber: QColor = QColor("#8A3324")
    butter: QColor = QColor("#FFFD74")
    butter_toast: QColor = QColor("#D2AD84")
    buttercream: QColor = QColor("#EFE0CD")
    buttermilk: QColor = QColor("#FDEFB2")
    butterscotch_orange: QColor = QColor("#D99151")
    buttery_brown: QColor = QColor("#F1EBDA")
    byzantine_blue: QColor = QColor("#3457D5")
    byzantium: QColor = QColor("#702963")
    cadet_gray: QColor = QColor("#91A3B0")
    cadmium_green: QColor = QColor("#006B3C")
    cadmium_orange: QColor = QColor("#E6812F")
    cadmium_red: QColor = QColor("#E30022")
    cadmium_yellow_lemon: QColor = QColor("#FFF23E")
    cafe_americano: QColor = QColor("#362819")
    cafe_au_lait: QColor = QColor("#A67B5B")
    cafe_noir: QColor = QColor("#4B3621")
    cake_batter: QColor = QColor("#F0EDDB")
    calm_orange: QColor = QColor("#FAB972")
    cambridge_blue: QColor = QColor("#A3C1AD")
    camel: QColor = QColor("#C69F59")
    cameo_pink: QColor = QColor("#EFBBCC")
    camouflage_gray: QColor = QColor("#545851")
    camouflage_green: QColor = QColor("#78866B")
    camouflage_yellow: QColor = QColor("#978942")
    canadian_maple: QColor = QColor("#CAB266")
    canary_yellow: QColor = QColor("#FFEF00")
    candy: QColor = QColor("#D21404")
    candy_apple: QColor = QColor("#FF0800")
    candy_pink: QColor = QColor("#E4717A")
    cantaloupe: QColor = QColor("#FDA172")
    canvas_brown: QColor = QColor("#BB8855")
    capri_blue: QColor = QColor("#00BFFF")
    caramel: QColor = QColor("#AF6F09")
    caramel_apple: QColor = QColor("#B87A59")
    carbon_black: QColor = QColor("#0C0A00")
    carbon_gray: QColor = QColor("#625D5D")
    cardinal: QColor = QColor("#8A244E")
    cardinal_red: QColor = QColor("#C41E3A")
    cardinals_red: QColor = QColor("#BD2031")
    careys: QColor = QColor("#D8A8A8")
    caribbean_green: QColor = QColor("#00CC99")
    carmine: QColor = QColor("#960018")
    carnal_brown: QColor = QColor("#BB8866")
    carnation: QColor = QColor("#FFA6C9")
    carnelian_red: QColor = QColor("#B31B1B")
    carob: QColor = QColor("#362511")
    carolina_blue: QColor = QColor("#4B9CD3")
    carolina_plum: QColor = QColor("#9B84A1")
    carrot: QColor = QColor("#ED7117")
    carrot_orange: QColor = QColor("#ED9121")
    cascading_white: QColor = QColor("#F7F5F6")
    cashmere: QColor = QColor("#D1B399")
    castleton_green: QColor = QColor("#00563B")
    cedar_brown: QColor = QColor("#4A3728")
    celadon: QColor = QColor("#ACE1AF")
    celeste: QColor = QColor("#B2FFFF")
    cement_gray: QColor = QColor("#858A7E")
    ceramic: QColor = QColor("#FCFFF9")
    cerise_pink: QColor = QColor("#EC3B83")
    cerise_red: QColor = QColor("#DE3163")
    cerulean: QColor = QColor("#0492C2")
    cerulean_blue: QColor = QColor("#2A52BE")
    cg_red: QColor = QColor("#E03C31")
    chai: QColor = QColor("#B1832F")
    chameleon_gray: QColor = QColor("#686965")
    chamoisee: QColor = QColor("#A0785A")
    chamomile: QColor = QColor("#DAC395")
    champagne: QColor = QColor("#F7E7CE")
    champagne_pink: QColor = QColor("#F6E1D3")
    charcoal: QColor = QColor("#36454F")
    charcoal_gray: QColor = QColor("#232023")
    charleston_green: QColor = QColor("#232B2B")
    chartreuse: QColor = QColor("#7FFF00")
    chartreuse_traditional: QColor = QColor("#DFFF00")
    chateau: QColor = QColor("#48A860")
    cheese_orange: QColor = QColor("#FFA600")
    chelsea_blue: QColor = QColor("#034694")
    cherry_blossom: QColor = QColor("#FFB7C5")
    cherry_pink: QColor = QColor("#DE3163")
    cherry_red: QColor = QColor("#990F02")
    chestnut: QColor = QColor("#CD5C5C")
    chetwode: QColor = QColor("#F0FFF0")
    chicory: QColor = QColor("#A78658")
    chiefs_red: QColor = QColor("#E31837")
    chiffon: QColor = QColor("#FFFACD")
    chili_red: QColor = QColor("#C21807")
    china_rose: QColor = QColor("#A8516E")
    chinese_green: QColor = QColor("#D0DB61")
    chinese_orange: QColor = QColor("#EB6841")
    chinese_violet: QColor = QColor("#856088")
    chinese_yellow: QColor = QColor("#FFB200")
    chlorine_blue: QColor = QColor("#0CAFFF")
    chlorophyll_green: QColor = QColor("#4AFF00")
    chocolate: QColor = QColor("#7B3F00")
    chocolate_cosmos_red: QColor = QColor("#58111A")
    chocolate_orange: QColor = QColor("#D2691E")
    christmas_green: QColor = QColor("#00873E")
    chrome: QColor = QColor("#DBE2E9")
    cider: QColor = QColor("#B56727")
    cider_orange: QColor = QColor("#D8A175")
    cincinnati_red: QColor = QColor("#C6011F")
    cinereous: QColor = QColor("#98817B")
    cinnabar_red: QColor = QColor("#E44D2E")
    cinnamon_brown: QColor = QColor("#652A0E")
    cinnamon_toast: QColor = QColor("#8D7D77")
    citrine: QColor = QColor("#E4D00A")
    citrus_brown: QColor = QColor("#D7C275")
    clay: QColor = QColor("#80400B")
    cloud: QColor = QColor("#C5C6D0")
    clover_lime: QColor = QColor("#FCE883")
    coal: QColor = QColor("#0C0908")
    cobalt: QColor = QColor("#1338BE")
    cobalt_blue: QColor = QColor("#0047AB")
    coconut: QColor = QColor("#FFF1E6")
    coconut_butter: QColor = QColor("#F2EFE1")
    coconut_milk: QColor = QColor("#FFF1E6")
    coconut_white: QColor = QColor("#E9EDF6")
    coffee: QColor = QColor("#6F4E37")
    coin: QColor = QColor("#9897A9")
    coke_red: QColor = QColor("#F40009")
    cola: QColor = QColor("#F9EDEB")
    college_navy: QColor = QColor("#002244")
    columbia_blue: QColor = QColor("#B9D9EB")
    colza_yellow: QColor = QColor("#EEB700")
    commercial_white: QColor = QColor("#EDECE6")
    cool_black: QColor = QColor("#151922")
    cool_gray: QColor = QColor("#9090C0")
    copper: QColor = QColor("#B87333")
    copper_orange: QColor = QColor("#BB772E")
    copper_red: QColor = QColor("#D97448")
    copper_rose: QColor = QColor("#996666")
    coquelicot_red: QColor = QColor("#FF3800")
    coral: QColor = QColor("#FF7F50")
    coral_pink: QColor = QColor("#FE7D6A")
    coral_red: QColor = QColor("#FF7F50")
    cordovan_red: QColor = QColor("#893F45")
    corn: QColor = QColor("#FBEC5D")
    corn_green: QColor = QColor("#71A92C")
    cornflower_blue: QColor = QColor("#6495ED")
    cornsilk: QColor = QColor("#FFF8DC")
    cosmic_latte: QColor = QColor("#FFF8E7")
    cotton: QColor = QColor("#FBFBF9")
    cotton_ball: QColor = QColor("#F2F7FD")
    cotton_candy: QColor = QColor("#FFBCD9")
    cough_mixture: QColor = QColor("#F6F4F1")
    crayola_blue: QColor = QColor("#1F75FE")
    crayola_orange: QColor = QColor("#FF7538")
    crayola_red: QColor = QColor("#EE204D")
    crayola_yellow_green: QColor = QColor("#ACDF87")
    cream: QColor = QColor("#FFFDD0")
    crimson: QColor = QColor("#B80F0A")
    crocodile: QColor = QColor("#607D3B")
    croissant: QColor = QColor("#C4AB86")
    crow: QColor = QColor("#0D0907")
    cup_cake: QColor = QColor("#EBF6F7")
    currant: QColor = QColor("#680C07")
    cyan_blue: QColor = QColor("#00FFFF")
    cyber_grape_purple: QColor = QColor("#58427C")
    cyber_yellow: QColor = QColor("#FFD300")
    cynical_black: QColor = QColor("#171717")
    daffodil_yellow: QColor = QColor("#E28D00")
    dahlia_yellow_ral: QColor = QColor("#F39B1B")
    daisy: QColor = QColor("#FAFAFA")
    daisy_yellow: QColor = QColor("#F8DC75")
    dallas_cowboys_blue: QColor = QColor("#041E42")
    dandelion: QColor = QColor("#F0E130")
    dark_beige: QColor = QColor("#AC9362")
    dark_black: QColor = QColor("#010203")
    dark_blue: QColor = QColor("#00008B")
    dark_brown: QColor = QColor("#654321")
    dark_byzantium: QColor = QColor("#5D3954")
    dark_charcoal: QColor = QColor("#333333")
    dark_chestnut: QColor = QColor("#986960")
    dark_chocolate: QColor = QColor("#2E1503")
    dark_coral: QColor = QColor("#CD5B45")
    dark_gray: QColor = QColor("#A9A9A9")
    dark_jungle_green: QColor = QColor("#1A2421")
    dark_khaki: QColor = QColor("#9B8F55")
    dark_lemon_lime: QColor = QColor("#76BA1B")
    dark_magenta: QColor = QColor("#8B008B")
    dark_moss_green: QColor = QColor("#4A5D23")
    dark_mulberry: QColor = QColor("#4C0121")
    dark_olive_green: QColor = QColor("#556B2F")
    dark_orange: QColor = QColor("#FF8C00")
    dark_orchid: QColor = QColor("#9932CC")
    dark_pastel_red: QColor = QColor("#C23B22")
    dark_pink: QColor = QColor("#E75480")
    dark_purple: QColor = QColor("#301934")
    dark_raisin: QColor = QColor("#1A0F0F")
    dark_red: QColor = QColor("#8B0000")
    dark_salmon: QColor = QColor("#E9967A")
    dark_sand: QColor = QColor("#9F7D4F")
    dark_sapphire: QColor = QColor("#082567")
    dark_sea_green: QColor = QColor("#8FBC8F")
    dark_sienna: QColor = QColor("#3C1414")
    dark_silver: QColor = QColor("#746D69")
    dark_slate: QColor = QColor("#3E3D53")
    dark_slate_gray: QColor = QColor("#2F4F4F")
    dark_spring_green: QColor = QColor("#177245")
    dark_terra_cotta: QColor = QColor("#CC4E5C")
    dark_topaz_orange: QColor = QColor("#EABD8C")
    dark_turquoise: QColor = QColor("#00CED1")
    dark_vanilla: QColor = QColor("#D1BEA8")
    dark_violet: QColor = QColor("#9400D3")
    dark_wood: QColor = QColor("#855E42")
    dark_yellow: QColor = QColor("#D5B60A")
    dartmouth_green: QColor = QColor("#00703C")
    davys_gray: QColor = QColor("#555555")
    debian_red: QColor = QColor("#D70A53")
    decorators_white: QColor = QColor("#ECEFEC")
    deep_black: QColor = QColor("#050203")
    deep_brown: QColor = QColor("#410200")
    deep_chrome_yellow_ferrario: QColor = QColor("#FFB045")
    deep_coffee: QColor = QColor("#704241")
    deep_green: QColor = QColor("#056608")
    deep_jungle_green: QColor = QColor("#004B49")
    deep_lilac: QColor = QColor("#B65FCF")
    deep_magenta: QColor = QColor("#CC00CC")
    deep_napoli_yellow: QColor = QColor("#FFCF99")
    deep_pink: QColor = QColor("#FF1493")
    deep_pruce: QColor = QColor("#A95C68")
    deep_purple: QColor = QColor("#524F81")
    deep_ruby_purple: QColor = QColor("#843F5B")
    deep_saffron: QColor = QColor("#FFA52C")
    deep_space_sparkle: QColor = QColor("#4A646C")
    deep_taupe: QColor = QColor("#7B6660")
    deep_wine: QColor = QColor("#2C041C")
    deep_yellow: QColor = QColor("#EACE09")
    deer: QColor = QColor("#BA8759")
    delft_blue: QColor = QColor("#1F305E")
    delicate_white: QColor = QColor("#F1F2EE")
    dell_blue: QColor = QColor("#0076CE")
    denim: QColor = QColor("#151E3D")
    denim_blue: QColor = QColor("#1560BD")
    desert_gray: QColor = QColor("#B8A487")
    desert_orange: QColor = QColor("#ED8E4A")
    desert_sand_orange: QColor = QColor("#EDC9AF")
    desire: QColor = QColor("#EA3C53")
    dijon: QColor = QColor("#C49102")
    dim_gray: QColor = QColor("#696969")
    dirt: QColor = QColor("#9B7653")
    dodgers_blue: QColor = QColor("#005A9C")
    dollar_bill: QColor = QColor("#85BB65")
    dolphins_aqua: QColor = QColor("#008E97")
    dorian_gray: QColor = QColor("#ACA79E")
    dove: QColor = QColor("#7C6E7F")
    dragon_fire_orange: QColor = QColor("#FD652D")
    dried_moss: QColor = QColor("#CCB97E")
    duck_blue: QColor = QColor("#007791")
    duke_blue: QColor = QColor("#012169")
    dull_black: QColor = QColor("#161616")
    dull_brown: QColor = QColor("#876E4B")
    dull_gray: QColor = QColor("#8B837E")
    dull_orange: QColor = QColor("#DB804E")
    dull_purple: QColor = QColor("#9861A5")
    dull_yellow: QColor = QColor("#DECA57")
    durian_yellow: QColor = QColor("#E1BD27")
    dust_storm: QColor = QColor("#E5CCC9")
    dusty_gray: QColor = QColor("#B29CA2")
    dusty_orange: QColor = QColor("#E27A53")
    dusty_yellow: QColor = QColor("#D4CC9A")
    dutch_orange: QColor = QColor("#FF9B00")
    dutch_white: QColor = QColor("#EFDFBB")
    early_bird: QColor = QColor("#CEA2FD")
    earth_yellow: QColor = QColor("#E1A95F")
    east_carolina_university_purple: QColor = QColor("#592A8A")
    ebony: QColor = QColor("#555D50")
    echo: QColor = QColor("#898D86")
    ecru: QColor = QColor("#C2B280")
    eerie_black: QColor = QColor("#1B1B1B")
    egg_nog: QColor = QColor("#FAE29C")
    eggplant: QColor = QColor("#311432")
    eggshell: QColor = QColor("#F0EAD6")
    egyptian_blue: QColor = QColor("#1034A6")
    electric_black: QColor = QColor("#292929")
    electric_blue: QColor = QColor("#7DF9FF")
    electric_crimson: QColor = QColor("#FF003F")
    electric_gray: QColor = QColor("#A5ADAD")
    electric_indigo: QColor = QColor("#6F00FF")
    electric_lavender: QColor = QColor("#E39FF6")
    electric_purple: QColor = QColor("#BF00FF")
    electric_violet: QColor = QColor("#8F00FF")
    electric_yellow: QColor = QColor("#FFFF33")
    embassy_purple: QColor = QColor("#3B343C")
    emerald: QColor = QColor("#028A0F")
    emerald_green: QColor = QColor("#50C878")
    eminence: QColor = QColor("#6C3082")
    empire_yellow: QColor = QColor("#F7D000")
    english_red: QColor = QColor("#AB4B52")
    english_violet: QColor = QColor("#563C5C")
    everton_blue: QColor = QColor("#003399")
    facebook_blue: QColor = QColor("#1877F2")
    faded_orange: QColor = QColor("#CD946A")
    faded_purple: QColor = QColor("#795F80")
    faded_yellow: QColor = QColor("#ECE3A1")
    fairy_tale: QColor = QColor("#F2C1D1")
    falcons_red: QColor = QColor("#A71930")
    fall_harvest: QColor = QColor("#A78A59")
    falu_red: QColor = QColor("#801818")
    fandango: QColor = QColor("#B53389")
    fashion_black: QColor = QColor("#060403")
    fashion_gray: QColor = QColor("#8E8D8C")
    fawn: QColor = QColor("#C8A951")
    feather_brown: QColor = QColor("#B8AD9E")
    feather_white: QColor = QColor("#E7EAE5")
    fed_ex_purple: QColor = QColor("#660099")
    feldgrau: QColor = QColor("#4D5D53")
    fern: QColor = QColor("#5DBB63")
    fern_green: QColor = QColor("#4F7942")
    ferrari: QColor = QColor("#FF2800")
    fire: QColor = QColor("#DD571C")
    fire_brick: QColor = QColor("#B22222")
    fire_engine_red: QColor = QColor("#CE2029")
    fire_yellow: QColor = QColor("#FEDE17")
    firewood: QColor = QColor("#CEBAA8")
    flame: QColor = QColor("#E34A27")
    flame_red: QColor = QColor("#E25822")
    flamingo: QColor = QColor("#FC8EAC")
    flax: QColor = QColor("#EEDC82")
    flint: QColor = QColor("#7E7D9C")
    floral_white: QColor = QColor("#FFFAF0")
    fluorescent_blue: QColor = QColor("#15F4EE")
    fluorescent_yellow: QColor = QColor("#CCFF00")
    fog: QColor = QColor("#655967")
    folly_red: QColor = QColor("#FF004F")
    ford_blue: QColor = QColor("#2C3968")
    forest_green: QColor = QColor("#0B6623")
    fortune_cookie: QColor = QColor("#E0C5A1")
    fossil: QColor = QColor("#787276")
    french_beige: QColor = QColor("#A67B50")
    french_fuchsia: QColor = QColor("#FD3F92")
    french_gray: QColor = QColor("#B4B19A")
    french_lime: QColor = QColor("#9EFD38")
    french_mauve: QColor = QColor("#D473D4")
    french_pink: QColor = QColor("#F64A8A")
    french_puce_red: QColor = QColor("#4E1609")
    frost: QColor = QColor("#FCFBFC")
    frost_black: QColor = QColor("#191C20")
    fuchsia: QColor = QColor("#FF00FF")
    fuchsia_blue: QColor = QColor("#9C51B6")
    fulvous: QColor = QColor("#E68000")
    fun: QColor = QColor("#007848")
    fuzzy_wuzzy_red: QColor = QColor("#CC6666")
    gainsboro: QColor = QColor("#DCDCDC")
    gamboge: QColor = QColor("#E89611")
    garnet: QColor = QColor("#610C04")
    garnet_red: QColor = QColor("#733635")
    general_motors_blue: QColor = QColor("#073980")
    gentle_gray: QColor = QColor("#DADADA")
    ghost_white: QColor = QColor("#F8F8FF")
    giants_orange: QColor = QColor("#FE5A1D")
    gin: QColor = QColor("#D8E4BC")
    ginger: QColor = QColor("#B06500")
    ginger_ale: QColor = QColor("#C9A86A")
    ginger_orange: QColor = QColor("#B06500")
    gingerbread: QColor = QColor("#5E2C04")
    girl_scout_green: QColor = QColor("#00AE58")
    glaucous: QColor = QColor("#6082B6")
    glaucous_blue: QColor = QColor("#6082B6")
    glitter: QColor = QColor("#E6E8FA")
    glossy_black: QColor = QColor("#252324")
    glossy_gold: QColor = QColor("#FAE481")
    glossy_gray: QColor = QColor("#AFADA9")
    glossy_orange: QColor = QColor("#E78400")
    go_green: QColor = QColor("#00AB66")
    gold: QColor = QColor("#FFD700")
    gold_pink: QColor = QColor("#E6C7C2")
    golden_brown: QColor = QColor("#996515")
    golden_fog: QColor = QColor("#F0EAD2")
    golden_gate_bridge_orange: QColor = QColor("#C0362C")
    golden_poppy: QColor = QColor("#FCC200")
    golden_yellow_ral: QColor = QColor("#DD9F00")
    goldenrod: QColor = QColor("#DAA520")
    goldenrod_orange: QColor = QColor("#DAA520")
    goldfish_orange: QColor = QColor("#FF9913")
    good_tax: QColor = QColor("#C9A0FF")
    gothic_grape: QColor = QColor("#120321")
    granite: QColor = QColor("#676767")
    granny_smith_apple: QColor = QColor("#A8E4A0")
    granola: QColor = QColor("#D6B85A")
    grape: QColor = QColor("#663046")
    grape_color: QColor = QColor("#6F2DA8")
    grapefruit_pink: QColor = QColor("#E0707C")
    graphite: QColor = QColor("#594D5B")
    graphite_black_ral: QColor = QColor("#27292B")
    graphite_gray: QColor = QColor("#4B4E53")
    gray: QColor = QColor("#808080")
    gray_brown: QColor = QColor("#7F7053")
    gray_cloud: QColor = QColor("#B6B6B4")
    gray_cmyk: QColor = QColor("#939598")
    gray_dolphin: QColor = QColor("#5C5858")
    gray_goose: QColor = QColor("#D1D0CE")
    gray_green: QColor = QColor("#5E716A")
    gray_owl: QColor = QColor("#D5D5CD")
    gray_wolf: QColor = QColor("#504A4B")
    grayish_brown: QColor = QColor("#7A6A4F")
    grease: QColor = QColor("#080806")
    greek_orange: QColor = QColor("#E06E04")
    greek_villa: QColor = QColor("#F0EBE3")
    green: QColor = QColor("#3CB043")
    green_lizard: QColor = QColor("#A7F432")
    green_munsell: QColor = QColor("#00A877")
    green_screen: QColor = QColor("#00B140")
    green_yellow: QColor = QColor("#ADFF2F")
    greenish_yellow: QColor = QColor("#EEEA62")
    gun_smoke_gray: QColor = QColor("#8A868E")
    gunmetal: QColor = QColor("#2A3439")
    guppie_green: QColor = QColor("#00FF7F")
    half_and_half: QColor = QColor("#FFFFE4")
    halloween_orange: QColor = QColor("#EE5921")
    harbor_gray: QColor = QColor("#C7C6C1")
    hardwood: QColor = QColor("#937A62")
    harlequin: QColor = QColor("#3FFF00")
    harp: QColor = QColor("#EBF5F0")
    harvard_crimson: QColor = QColor("#A51C30")
    hawaiian_brown: QColor = QColor("#99522C")
    hazel_wood: QColor = QColor("#C9BB8E")
    hazelnut: QColor = QColor("#BDA55D")
    heather: QColor = QColor("#9AA297")
    heavy_brown: QColor = QColor("#73624A")
    heavy_gray: QColor = QColor("#050505")
    heliotrope: QColor = QColor("#DF73FF")
    hibiscus: QColor = QColor("#B43757")
    hickory: QColor = QColor("#371D10")
    hippie: QColor = QColor("#A84860")
    hit: QColor = QColor("#FFA878")
    hog_bristle_quarter: QColor = QColor("#EBE4D8")
    hollywood_cerise: QColor = QColor("#F400A1")
    hollywood_cerise_red: QColor = QColor("#F400A1")
    honey: QColor = QColor("#FFC30B")
    honey_brown: QColor = QColor("#BA9238")
    honey_orange: QColor = QColor("#FFBD31")
    honeydew: QColor = QColor("#F0FFF0")
    hookers_green: QColor = QColor("#49796B")
    hopbush: QColor = QColor("#D05FAD")
    hot_orange: QColor = QColor("#FF6E00")
    hot_pink: QColor = QColor("#FF69B4")
    hot_purple: QColor = QColor("#A420D0")
    hot_yellow: QColor = QColor("#FFDA00")
    hunter_green: QColor = QColor("#355E3B")
    huntington_white: QColor = QColor("#F1F0E5")
    hunyadi_orange: QColor = QColor("#E8AC41")
    hunyadi_yellow: QColor = QColor("#E8AC41")
    husker_red: QColor = QColor("#E41C38")
    ibm_blue: QColor = QColor("#0530AD")
    ice_blue: QColor = QColor("#99FFFF")
    ice_cube: QColor = QColor("#E3E4E0")
    ice_gray: QColor = QColor("#BEC3C6")
    iced_coffee: QColor = QColor("#B18F6A")
    icterine: QColor = QColor("#FCF75E")
    iguana_green: QColor = QColor("#71BC78")
    imperial_blue: QColor = QColor("#005A92")
    imperial_purple: QColor = QColor("#602F6B")
    imperial_red: QColor = QColor("#ED2939")
    inchworm: QColor = QColor("#B2EC5D")
    independence_blue: QColor = QColor("#4C516D")
    india_green: QColor = QColor("#138808")
    india_saffron: QColor = QColor("#FF9933")
    indian_red: QColor = QColor("#CD5C5C")
    indian_yellow: QColor = QColor("#E3A857")
    indigo: QColor = QColor("#4B0082")
    indigo_blue: QColor = QColor("#3F00FF")
    indigo_dye_blue: QColor = QColor("#00416A")
    ink: QColor = QColor("#070504")
    ink_black_ral: QColor = QColor("#212122")
    insolent_purple: QColor = QColor("#682E3C")
    intel_blue: QColor = QColor("#0071C5")
    international_klein_blue: QColor = QColor("#002FA7")
    international_orange: QColor = QColor("#FF4F00")
    iridium: QColor = QColor("#3D3C3A")
    iris: QColor = QColor("#9867C5")
    iris_blue: QColor = QColor("#5A4FCF")
    irish_green: QColor = QColor("#009A44")
    irish_orange: QColor = QColor("#F5883F")
    iron: QColor = QColor("#A59C94")
    iron_black: QColor = QColor("#343432")
    isabelline: QColor = QColor("#F4F0EC")
    island_spice: QColor = QColor("#FFFCEC")
    ivory: QColor = QColor("#FFFFF0")
    ivory_black: QColor = QColor("#231F20")
    jade: QColor = QColor("#000302")
    jade_green: QColor = QColor("#00A86B")
    jam: QColor = QColor("#67032F")
    japanese_carmine: QColor = QColor("#9D2933")
    japanese_violet_purple: QColor = QColor("#5B3256")
    jasper_orange: QColor = QColor("#E89149")
    jasper_red: QColor = QColor("#D73B3E")
    jazzberry_jam: QColor = QColor("#67032F")
    jelly_bean_red: QColor = QColor("#DA614E")
    jet: QColor = QColor("#343434")
    jet_black: QColor = QColor("#030303")
    jet_gray: QColor = QColor("#616D7E")
    john_deere_green: QColor = QColor("#367C2B")
    jungle_green: QColor = QColor("#29AB87")
    juniper: QColor = QColor("#3A5311")
    kaitoke: QColor = QColor("#004830")
    kalamata: QColor = QColor("#705160")
    kc_royals_blue: QColor = QColor("#004687")
    kelly_green: QColor = QColor("#4CBB17")
    kenyan_copper_red: QColor = QColor("#7C1C05")
    khaki: QColor = QColor("#C3B091")
    kikyo_iru: QColor = QColor("#5D3F6A")
    kingfisher_daisy: QColor = QColor("#653780")
    knockout_pink: QColor = QColor("#FF3EA5")
    kobicha: QColor = QColor("#6B4423")
    koi_orange: QColor = QColor("#D15837")
    kokushoku: QColor = QColor("#171412")
    kombu_green: QColor = QColor("#354230")
    ksu_purple: QColor = QColor("#512888")
    kurotobi: QColor = QColor("#351E1C")
    la_salle_green: QColor = QColor("#087830")
    lace: QColor = QColor("#F8F2ED")
    lady: QColor = QColor("#F0C0A8")
    laguna: QColor = QColor("#F8E473")
    laid_back_gray: QColor = QColor("#B4AFA9")
    lakers_purple: QColor = QColor("#552582")
    lamp_black: QColor = QColor("#2E473B")
    languid_lavender: QColor = QColor("#D6CADD")
    lapis: QColor = QColor("#2832C2")
    lapis_blue: QColor = QColor("#26619C")
    laser_lemon: QColor = QColor("#FEFE22")
    latte: QColor = QColor("#C5A582")
    laurel_green: QColor = QColor("#A9BA9D")
    lava: QColor = QColor("#808588")
    lava_orange: QColor = QColor("#F76806")
    lava_red: QColor = QColor("#CF1020")
    lavender_blush: QColor = QColor("#FFF0F5")
    lavender_gray: QColor = QColor("#C4C3D0")
    lavender_rose: QColor = QColor("#FBA0E3")
    lawn_green: QColor = QColor("#7CFC00")
    lead: QColor = QColor("#3F3F4E")
    leather: QColor = QColor("#0B0705")
    lemon_chiffon: QColor = QColor("#FFFACD")
    lemon_curry: QColor = QColor("#CCA01D")
    lemon_yellow: QColor = QColor("#FDFF00")
    lemonade: QColor = QColor("#F2DBE7")
    lemonade_crepe: QColor = QColor("#F2B8C6")
    lexicon: QColor = QColor("#E7EAEA")
    lexicon_quarter: QColor = QColor("#F2F2F1")
    liberty_blue: QColor = QColor("#545AA7")
    licorice: QColor = QColor("#1A1110")
    light_blush: QColor = QColor("#F1ABB9")
    light_brick: QColor = QColor("#FB607F")
    light_brown: QColor = QColor("#B5651D")
    light_chrome_yellow: QColor = QColor("#FFCA37")
    light_copper_orange: QColor = QColor("#F6CBA0")
    light_coral: QColor = QColor("#F08080")
    light_crimson: QColor = QColor("#F56991")
    light_deep_pink: QColor = QColor("#FF5CCD")
    light_goldenrod: QColor = QColor("#FAFAD2")
    light_gray: QColor = QColor("#D3D3D3")
    light_gunmetal: QColor = QColor("#344148")
    light_hot_pink: QColor = QColor("#FFB3DE")
    light_khaki: QColor = QColor("#F0E68C")
    light_orange: QColor = QColor("#FED8B1")
    light_pink: QColor = QColor("#FFB6C1")
    light_red_ochre: QColor = QColor("#E97451")
    light_saffron: QColor = QColor("#FFE797")
    light_salmon: QColor = QColor("#FDAB9F")
    light_slate_gray: QColor = QColor("#6D7B8D")
    light_taupe: QColor = QColor("#B38B6D")
    light_turquoise: QColor = QColor("#AFEEEE")
    light_yellow: QColor = QColor("#FFFFE0")
    lightsaber_green: QColor = QColor("#2FF924")
    lilac: QColor = QColor("#B65FCF")
    lime: QColor = QColor("#AEF359")
    lime_green: QColor = QColor("#C7EA46")
    limerick: QColor = QColor("#9DC209")
    linen: QColor = QColor("#F3EAD3")
    link_white: QColor = QColor("#ECF3F9")
    lip_pink: QColor = QColor("#DBAC98")
    lipstick: QColor = QColor("#9B1003")
    lipstick_stain: QColor = QColor("#8E4785")
    liserian_purple: QColor = QColor("#DE6FA1")
    little_boy_blue: QColor = QColor("#6CA0DC")
    little_princess: QColor = QColor("#E9DCE5")
    liver: QColor = QColor("#534B4F")
    liverpool_red: QColor = QColor("#C8102E")
    long_distance: QColor = QColor("#6F456E")
    lovely_purple: QColor = QColor("#7F38EC")
    lowes_blue: QColor = QColor("#004792")
    luminous_bright_orange_ral: QColor = QColor("#FFB700")
    lust_red: QColor = QColor("#E62020")
    luxury_black: QColor = QColor("#060D0D")
    luxury_purple: QColor = QColor("#743089")
    macaroon: QColor = QColor("#F9E076")
    madder_red: QColor = QColor("#A50021")
    magenta: QColor = QColor("#FF00FF")
    magenta_process: QColor = QColor("#E11584")
    magenta_purple: QColor = QColor("#A32CC4")
    magic_blue: QColor = QColor("#0077C0")
    magnolia: QColor = QColor("#F8F4FF")
    mahogany: QColor = QColor("#C04000")
    mahogany_orange: QColor = QColor("#C04000")
    maize_yellow_ral: QColor = QColor("#DB9100")
    majorelle_blue: QColor = QColor("#6050DC")
    malachite: QColor = QColor("#0BDA51")
    manchester_united_red: QColor = QColor("#DA291C")
    mango_tango: QColor = QColor("#FB8842")
    mantis: QColor = QColor("#74C365")
    maple_sugar: QColor = QColor("#C9A38D")
    maple_syrup: QColor = QColor("#BB9351")
    mardi_gras: QColor = QColor("#880085")
    marengo: QColor = QColor("#4C5866")
    marian_blue: QColor = QColor("#E1EBEE")
    marigold: QColor = QColor("#FCB810")
    marigold_orange: QColor = QColor("#EAA221")
    marmalade: QColor = QColor("#D16002")
    maroon: QColor = QColor("#800000")
    mars_brown: QColor = QColor("#AD6242")
    matte_purple: QColor = QColor("#392A48")
    matte_silver: QColor = QColor("#A5A5A8")
    matte_yellow: QColor = QColor("#EDD94C")
    mauve: QColor = QColor("#E0B0FF")
    mauveine: QColor = QColor("#8D029B")
    mauvelous: QColor = QColor("#EF98AA")
    mavericks_blue: QColor = QColor("#00538C")
    maximum_green: QColor = QColor("#4C9A2A")
    maximum_green_yellow: QColor = QColor("#D9E650")
    maximum_yellow: QColor = QColor("#FAFA37")
    maya_blue: QColor = QColor("#73C2FB")
    mecca_orange: QColor = QColor("#BD5745")
    medium_blue: QColor = QColor("#0000CD")
    medium_brown: QColor = QColor("#7F5112")
    medium_gray: QColor = QColor("#BEBEBE")
    medium_jungle_green: QColor = QColor("#1C352D")
    medium_orchid: QColor = QColor("#BA55D3")
    medium_purple: QColor = QColor("#9370DB")
    medium_purple_3: QColor = QColor("#8968CD")
    medium_sapphire: QColor = QColor("#2D5DA1")
    medium_sea_green: QColor = QColor("#3CB371")
    medium_slate_blue: QColor = QColor("#7B68EE")
    medium_spring_bud: QColor = QColor("#C9DC87")
    medium_spring_green: QColor = QColor("#00FA9A")
    medium_turquoise: QColor = QColor("#48D1CC")
    medium_vermilion: QColor = QColor("#CF5B2E")
    medium_vermillion: QColor = QColor("#D9603B")
    medium_violet_red: QColor = QColor("#C71585")
    mellow_yellow: QColor = QColor("#F8DE7E")
    melon: QColor = QColor("#F7BCAC")
    melon_orange: QColor = QColor("#DF9166")
    melon_yellow: QColor = QColor("#FF9C00")
    mercury: QColor = QColor("#E7E7E7")
    merigold: QColor = QColor("#FCAE1E")
    merino: QColor = QColor("#F2EBDD")
    merlot: QColor = QColor("#541E1B")
    metal: QColor = QColor("#0E0C0A")
    metallic_gray: QColor = QColor("#A8A9AD")
    metallic_orange: QColor = QColor("#E26310")
    metallic_purple: QColor = QColor("#520E7D")
    metropolis: QColor = QColor("#1A1A1A")
    mexican_pink: QColor = QColor("#E4007C")
    middle_green: QColor = QColor("#4D8C57")
    middle_purple: QColor = QColor("#D982B5")
    midnight: QColor = QColor("#2B1B17")
    midnight_green: QColor = QColor("#004953")
    mikado_yellow: QColor = QColor("#FFC40C")
    milano: QColor = QColor("#D95D67")
    milk: QColor = QColor("#FDFFF5")
    millennium_blue: QColor = QColor("#002244")
    mimosa: QColor = QColor("#FFF4BC")
    mimosa_orange: QColor = QColor("#FFCA4B")
    mineral_yellow: QColor = QColor("#D39C43")
    minion_yellow: QColor = QColor("#F5E050")
    mink: QColor = QColor("#888078")
    mint: QColor = QColor("#99EDC3")
    mint_cream: QColor = QColor("#F5FFFA")
    mint_green: QColor = QColor("#98FB98")
    misty_gray: QColor = QColor("#BCC2C2")
    misty_rose: QColor = QColor("#FFE4E1")
    mit_red: QColor = QColor("#A31F34")
    moccasin: QColor = QColor("#FFE4B5")
    mocha: QColor = QColor("#3C280D")
    mojave_desert: QColor = QColor("#C7B595")
    morning_blue: QColor = QColor("#8DA399")
    moss: QColor = QColor("#466D1D")
    moss_green: QColor = QColor("#8A9A5B")
    mountains_majesty: QColor = QColor("#9078C0")
    mountbatten_pink: QColor = QColor("#997A8D")
    mughal_green: QColor = QColor("#306030")
    mulberry: QColor = QColor("#C54B8C")
    munsell_blue: QColor = QColor("#0093AF")
    munsell_purple: QColor = QColor("#9F00C5")
    munsell_red: QColor = QColor("#F2003C")
    munsell_yellow: QColor = QColor("#EFCC00")
    murasaki_purple: QColor = QColor("#4F284B")
    mustard: QColor = QColor("#FEDC56")
    muted_yellow: QColor = QColor("#E0D268")
    myrtle_green: QColor = QColor("#317873")
    mythical_purple: QColor = QColor("#53277E")
    nadeshiko_pink: QColor = QColor("#F6ADC6")
    napier_green: QColor = QColor("#2A8000")
    nationals_red: QColor = QColor("#AB0003")
    natural_black: QColor = QColor("#07000B")
    natural_white: QColor = QColor("#EEECE6")
    navajo_white: QColor = QColor("#FFDEAD")
    navy: QColor = QColor("#0A1172")
    navy_blue: QColor = QColor("#000080")
    ncs_red: QColor = QColor("#C40233")
    neon_blue: QColor = QColor("#4D4DFF")
    neon_fuchsia: QColor = QColor("#FE4164")
    neon_green: QColor = QColor("#39FF14")
    neon_orange: QColor = QColor("#FFAD00")
    neon_pink: QColor = QColor("#FF6EC7")
    neon_yellow: QColor = QColor("#CFFF04")
    neutral_black: QColor = QColor("#0B0B0B")
    neutral_gray: QColor = QColor("#898E8C")
    nevada: QColor = QColor("#666A6D")
    new_york_pink: QColor = QColor("#DD8374")
    nickel: QColor = QColor("#727472")
    night: QColor = QColor("#0C090A")
    night_shadow: QColor = QColor("#1C1C1C")
    night_white: QColor = QColor("#E1E1DD")
    non_photo_blue: QColor = QColor("#A4DDED")
    northwestern_purple: QColor = QColor("#4E2A84")
    nude_pink: QColor = QColor("#DDC0B4")
    nyanza: QColor = QColor("#E9FFDB")
    oat: QColor = QColor("#DFC98A")
    obsidian: QColor = QColor("#020403")
    ocean: QColor = QColor("#016064")
    ocean_blue: QColor = QColor("#009DC4")
    ocean_purple: QColor = QColor("#6E2D91")
    ochre: QColor = QColor("#CC7722")
    ochre_orange: QColor = QColor("#C99700")
    ochre_yellow: QColor = QColor("#B5904B")
    off_black: QColor = QColor("#595652")
    office_green: QColor = QColor("#008000")
    oil: QColor = QColor("#3B3131")
    oklahoma_crimson: QColor = QColor("#841617")
    old_burgundy: QColor = QColor("#43302E")
    old_lace: QColor = QColor("#FDF5E6")
    old_lavender: QColor = QColor("#796878")
    old_moss_green: QColor = QColor("#867E36")
    old_rose: QColor = QColor("#C08081")
    old_silver: QColor = QColor("#848482")
    old_wood: QColor = QColor("#F0EEE4")
    old_yellow: QColor = QColor("#C8AF55")
    olive: QColor = QColor("#708238")
    olive_drab: QColor = QColor("#6B8E23")
    olive_drab_7: QColor = QColor("#3C341F")
    olive_gray: QColor = QColor("#A09F8D")
    olive_yellow_ral: QColor = QColor("#9D8F65")
    onyx: QColor = QColor("#353839")
    orange: QColor = QColor("#ED7014")
    orange_pale_orange: QColor = QColor("#FFDFBF")
    orange_paper_orange: QColor = QColor("#FEE8D6")
    orange_peel: QColor = QColor("#FF9F00")
    orange_red: QColor = QColor("#FF4500")
    orange_spice_orange: QColor = QColor("#D76B00")
    orange_white: QColor = QColor("#F8F9F5")
    orange_yellow: QColor = QColor("#FFAD01")
    orchid: QColor = QColor("#DA70D6")
    orioles_orange: QColor = QColor("#FB4F14")
    outer_space: QColor = QColor("#414A4C")
    outrageous_orange: QColor = QColor("#FF6E4A")
    outrageous_orange_red: QColor = QColor("#FF6E4A")
    oxblood_red: QColor = QColor("#800020")
    oxford_blue: QColor = QColor("#002147")
    oxford_gray: QColor = QColor("#8391A1")
    oyster: QColor = QColor("#F0D8D8")
    oyster_white: QColor = QColor("#E3DFD2")
    pacific_blue: QColor = QColor("#1CA9C9")
    pacific_pearl: QColor = QColor("#E8EAE6")
    pakistan_green: QColor = QColor("#006600")
    palatinate: QColor = QColor("#72246C")
    pale_goldenrod: QColor = QColor("#EEE8AA")
    pale_ivory: QColor = QColor("#FDF6E4")
    pale_lime_yellow: QColor = QColor("#DFE69F")
    pale_purple: QColor = QColor("#AA98A9")
    pale_smoke: QColor = QColor("#CAD4D1")
    pale_violet_red: QColor = QColor("#DB7093")
    pale_yellow: QColor = QColor("#FFFFA7")
    panda_black: QColor = QColor("#3C4748")
    pansy_purple: QColor = QColor("#78184A")
    pantone_artichoke_green: QColor = QColor("#4B6F44")
    pantone_blue: QColor = QColor("#0018A8")
    pantone_orange: QColor = QColor("#FE5000")
    paolo_veronese_green: QColor = QColor("#009B7D")
    papaya_whip: QColor = QColor("#FFEFD5")
    paper_white: QColor = QColor("#F7FCFE")
    paradise_pink: QColor = QColor("#E63E62")
    parakeet: QColor = QColor("#03C04A")
    parchment: QColor = QColor("#F1E9D2")
    paris_green: QColor = QColor("#50C878")
    paris_white: QColor = QColor("#C2D8D3")
    parmesan: QColor = QColor("#FDE992")
    passion_pink: QColor = QColor("#CE74A7")
    pastel_black: QColor = QColor("#1D1C1A")
    pastel_gray: QColor = QColor("#CCCDC6")
    pastel_orange: QColor = QColor("#FEBA4F")
    pastel_pink: QColor = QColor("#FFD1DC")
    pastel_red: QColor = QColor("#FF6961")
    pastel_yellow: QColor = QColor("#FFFE71")
    paynes_gray: QColor = QColor("#40404F")
    peach: QColor = QColor("#FAD1AF")
    peach_orange: QColor = QColor("#FFCC99")
    peacock: QColor = QColor("#022D36")
    peacock_blue: QColor = QColor("#005F69")
    peanut_brown: QColor = QColor("#795C34")
    pear: QColor = QColor("#D1E231")
    pearl: QColor = QColor("#FBFCF8")
    pearly_purple: QColor = QColor("#B768A2")
    pebble: QColor = QColor("#333333")
    pecan: QColor = QColor("#4A2511")
    penn_blue: QColor = QColor("#011F5B")
    penn_red: QColor = QColor("#990000")
    penny: QColor = QColor("#532915")
    pepper_orange: QColor = QColor("#E77D22")
    peppermint: QColor = QColor("#F1F9EC")
    perfect_gray: QColor = QColor("#A1A1A1")
    periwinkle: QColor = QColor("#CCCCFF")
    periwinkle_purple: QColor = QColor("#BE93E4")
    persian: QColor = QColor("#CA3433")
    persian_blue: QColor = QColor("#1C39BB")
    persian_green: QColor = QColor("#00A693")
    persian_orange: QColor = QColor("#D99058")
    persian_pink: QColor = QColor("#F77FBE")
    persian_red: QColor = QColor("#CA3433")
    persian_rose_red: QColor = QColor("#FE28A2")
    persimmon: QColor = QColor("#EC5800")
    peru: QColor = QColor("#CD853F")
    pewter: QColor = QColor("#696880")
    philippine_orange: QColor = QColor("#F26E01")
    phillies_red: QColor = QColor("#E81828")
    phlox: QColor = QColor("#DF00FF")
    photon_white: QColor = QColor("#F8F8E8")
    phthalo_blue: QColor = QColor("#000F89")
    phthalo_green: QColor = QColor("#123524")
    pickle: QColor = QColor("#597D35")
    picotee_blue: QColor = QColor("#2E2787")
    piggy_pink: QColor = QColor("#FDDDE6")
    pigment_magenta: QColor = QColor("#FF0090")
    pigment_red: QColor = QColor("#ED1C24")
    pine: QColor = QColor("#234F1E")
    pine_green: QColor = QColor("#01796F")
    pine_needle_color: QColor = QColor("#454D32")
    pink: QColor = QColor("#F699CD")
    pink_gray: QColor = QColor("#E7DADA")
    pink_pearl: QColor = QColor("#E7ACCF")
    pink_salt: QColor = QColor("#F8EEEC")
    pink_sherbet: QColor = QColor("#F78FA7")
    pinterest_red: QColor = QColor("#E60023")
    pistachio: QColor = QColor("#93C572")
    pitch: QColor = QColor("#030001")
    pitch_black: QColor = QColor("#27251F")
    pizza_edge: QColor = QColor("#9A2CA0")
    platinum: QColor = QColor("#E5E4E2")
    plum: QColor = QColor("#DDA0DD")
    polar_bear: QColor = QColor("#EAE9E0")
    polynesian_blue: QColor = QColor("#224C98")
    pomelo_white: QColor = QColor("#F9FFE3")
    pomp_and_power: QColor = QColor("#86608E")
    pompadour: QColor = QColor("#720058")
    popstar_red: QColor = QColor("#BE4F62")
    porcelain: QColor = QColor("#FFFEFC")
    porpoise: QColor = QColor("#4D4D5C")
    portland_orange_red: QColor = QColor("#FF5A36")
    powder: QColor = QColor("#FBFCFA")
    powder_blue: QColor = QColor("#B0E0E6")
    power_black: QColor = QColor("#0E0C01")
    power_gray: QColor = QColor("#707272")
    premium_black: QColor = QColor("#100E09")
    prince_charming: QColor = QColor("#493F5E")
    princeton_orange: QColor = QColor("#FF8F00")
    prismatic_legacy: QColor = QColor("#BA1607")
    prismatic_red: QColor = QColor("#D03D33")
    prismatic_reflections_shade: QColor = QColor("#FF3C28")
    prismatic_vermilion_renewal: QColor = QColor("#CA0123")
    prune_red: QColor = QColor("#701C1C")
    prussian_blue: QColor = QColor("#003153")
    psg_blue: QColor = QColor("#004170")
    psychedelic_yellow: QColor = QColor("#FFF53D")
    puce: QColor = QColor("#CC8899")
    pullman_green: QColor = QColor("#3B331C")
    pumpkin: QColor = QColor("#F5761A")
    pumpkin_orange: QColor = QColor("#FF7518")
    punch: QColor = QColor("#F25278")
    pure_orange: QColor = QColor("#FF8000")
    pure_silver: QColor = QColor("#C3C7C7")
    purple: QColor = QColor("#A32CC4")
    purple_4: QColor = QColor("#551A8B")
    purple_cmyk: QColor = QColor("#6A317F")
    purple_gray: QColor = QColor("#A7A6BA")
    purple_heart: QColor = QColor("#7442C8")
    purple_navy: QColor = QColor("#4E5180")
    purple_pizzazz: QColor = QColor("#FE4EDA")
    purple_taupe: QColor = QColor("#50404D")
    purple_violet: QColor = QColor("#47243C")
    purpureus: QColor = QColor("#9A4EAE")
    queen_blue: QColor = QColor("#436B95")
    radiant_yellow: QColor = QColor("#FC9E21")
    radical_red: QColor = QColor("#FF355E")
    radioactive: QColor = QColor("#FAE500")
    rainbow_yellow: QColor = QColor("#F5DE10")
    raisin: QColor = QColor("#290916")
    raisin_black: QColor = QColor("#242124")
    rajah: QColor = QColor("#FABA5F")
    raspberry: QColor = QColor("#D21F3C")
    raspberry_glace: QColor = QColor("#915F6D")
    raspberry_rose: QColor = QColor("#B3446C")
    raven: QColor = QColor("#050301")
    razzle_dazzle_rose: QColor = QColor("#FF33CC")
    razzmatazz: QColor = QColor("#E3256B")
    razzmic_berry_purple: QColor = QColor("#8D4E85")
    real_gold: QColor = QColor("#E2BF36")
    rebecca_purple: QColor = QColor("#663399")
    red: QColor = QColor("#FF0000")
    red_devil: QColor = QColor("#860111")
    red_fox_orange: QColor = QColor("#985629")
    red_orange_ral: QColor = QColor("#B6481C")
    red_sox: QColor = QColor("#BD3039")
    redwood: QColor = QColor("#A45A52")
    refresh_black: QColor = QColor("#111212")
    refresh_yellow: QColor = QColor("#FFE992")
    regalia_purple: QColor = QColor("#522D80")
    relax_yellow: QColor = QColor("#F3E496")
    reseda_green: QColor = QColor("#6C7C59")
    resolution_blue: QColor = QColor("#002387")
    retro_black: QColor = QColor("#1F201F")
    retro_white: QColor = QColor("#EDEAE2")
    retro_yellow: QColor = QColor("#F0E27B")
    revere_pewter: QColor = QColor("#CCC7B9")
    reversed_gray: QColor = QColor("#080808")
    rhino: QColor = QColor("#B9BBB6")
    rhythm: QColor = QColor("#777696")
    rice: QColor = QColor("#FAF5EF")
    rich_black_fogra29: QColor = QColor("#010B13")
    rich_gray: QColor = QColor("#1F2022")
    rifle_green: QColor = QColor("#444C38")
    ripe_plum: QColor = QColor("#410056")
    road_sign_green: QColor = QColor("#01735C")
    roasted_coffee: QColor = QColor("#4B371C")
    robin_egg_blue: QColor = QColor("#00CCCC")
    rocket_metallic: QColor = QColor("#8A7F8D")
    rose: QColor = QColor("#FC94AF")
    rose_bonbon: QColor = QColor("#F9429E")
    rose_ebony: QColor = QColor("#674846")
    rose_gold: QColor = QColor("#B76E79")
    rose_quartz: QColor = QColor("#AA98A9")
    rose_red: QColor = QColor("#C21E56")
    rose_taupe: QColor = QColor("#905D5D")
    rose_white: QColor = QColor("#FFFAFA")
    rosewood: QColor = QColor("#9E4244")
    rosewood_red: QColor = QColor("#65000B")
    rosy_pink: QColor = QColor("#FF66CC")
    rouge: QColor = QColor("#A94064")
    royal_orange: QColor = QColor("#FF9944")
    royal_periwinkle: QColor = QColor("#BE93D4")
    royal_yellow: QColor = QColor("#FADA5E")
    ruber: QColor = QColor("#CE4676")
    rubine_red: QColor = QColor("#D10056")
    ruby: QColor = QColor("#E0115F")
    ruddy_blue: QColor = QColor("#76ABDF")
    ruddy_pink: QColor = QColor("#E18E96")
    rufous: QColor = QColor("#A81C07")
    rufous_red: QColor = QColor("#A81C07")
    rumba_orange: QColor = QColor("#F06631")
    russet: QColor = QColor("#80461B")
    russet_red: QColor = QColor("#80461B")
    russian_green: QColor = QColor("#679267")
    russian_violet: QColor = QColor("#32174D")
    rust: QColor = QColor("#B7410E")
    rustic_black: QColor = QColor("#1B1A16")
    rustic_gray: QColor = QColor("#A6ABA5")
    rustic_orange: QColor = QColor("#D78C3D")
    rustic_purple: QColor = QColor("#593163")
    rustic_yellow: QColor = QColor("#E2C400")
    rusty_red: QColor = QColor("#DA2C43")
    sable: QColor = QColor("#060606")
    sacramento_green: QColor = QColor("#043927")
    saddle_brown: QColor = QColor("#8B4513")
    safety_orange: QColor = QColor("#FF6700")
    safety_yellow: QColor = QColor("#EED202")
    safflower_orange: QColor = QColor("#FDAE44")
    saffron: QColor = QColor("#F4C430")
    sage: QColor = QColor("#728C69")
    sage_gray: QColor = QColor("#B5BDA5")
    sage_green: QColor = QColor("#9DC183")
    salamander_orange: QColor = QColor("#F05E23")
    salmon: QColor = QColor("#FF9999")
    salmon_orange: QColor = QColor("#FF8066")
    sand: QColor = QColor("#F6D7B0")
    sand_dollar: QColor = QColor("#EDE8BA")
    sandcastle: QColor = QColor("#DAC17C")
    sandstone: QColor = QColor("#D67229")
    sandstorm: QColor = QColor("#ECD540")
    sangria: QColor = QColor("#4D0F28")
    sangria_purple: QColor = QColor("#4D0F28")
    sangria_red: QColor = QColor("#5E1916")
    sanguine_red: QColor = QColor("#BC3F4A")
    sap_green: QColor = QColor("#507D2A")
    sapphire: QColor = QColor("#52B2BF")
    sapphire_blue: QColor = QColor("#0F52BA")
    satin_sheen_gold: QColor = QColor("#CBA135")
    savoy_blue: QColor = QColor("#4B61D1")
    scarlet: QColor = QColor("#FF2400")
    schauss_pink: QColor = QColor("#FF91AF")
    school_bus_yellow: QColor = QColor("#FFD800")
    scorpion: QColor = QColor("#5E5E5E")
    screamin_green: QColor = QColor("#76FF7A")
    sea: QColor = QColor("#F09090")
    sea_gray: QColor = QColor("#9AA2A4")
    sea_green: QColor = QColor("#2E8B57")
    seafoam: QColor = QColor("#3DED97")
    seal: QColor = QColor("#818380")
    seance: QColor = QColor("#61346B")
    seashell: QColor = QColor("#FFF5EE")
    seaweed: QColor = QColor("#354A21")
    selective_yellow: QColor = QColor("#FFBA00")
    sepia: QColor = QColor("#704214")
    shades_on: QColor = QColor("#606060")
    shadow: QColor = QColor("#373737")
    shadow_white: QColor = QColor("#EEF1EA")
    shampoo: QColor = QColor("#FFCFF1")
    shamrock: QColor = QColor("#03AC13")
    shamrock_green: QColor = QColor("#009E60")
    sheen_green: QColor = QColor("#8FD400")
    shimmering_blush: QColor = QColor("#D98695")
    shiny_purple: QColor = QColor("#B941FF")
    shocking_pink: QColor = QColor("#FC0FC0")
    shortbread: QColor = QColor("#FBE790")
    shrek_green: QColor = QColor("#C4D300")
    sienna: QColor = QColor("#A0522D")
    signal_black_ral: QColor = QColor("#2B2B2C")
    signal_yellow: QColor = QColor("#F2A900")
    silk_gray: QColor = QColor("#B6B3A8")
    silver: QColor = QColor("#C0C0C0")
    silver_chalice: QColor = QColor("#ACACAC")
    silver_pink: QColor = QColor("#C4AEAD")
    silvery_pink: QColor = QColor("#DCB5B4")
    simply_white: QColor = QColor("#EFEDE3")
    sky: QColor = QColor("#63C5DA")
    sky_blue: QColor = QColor("#ADD8E6")
    slate: QColor = QColor("#26282A")
    slate_blue: QColor = QColor("#737CA1")
    slate_gray: QColor = QColor("#657383")
    slimy_green: QColor = QColor("#299617")
    smashed_pumpkin: QColor = QColor("#FD673A")
    smitten: QColor = QColor("#C84186")
    smoke: QColor = QColor("#738276")
    smokey_gray: QColor = QColor("#726E6D")
    smoky_black: QColor = QColor("#100C08")
    smyrna_purple: QColor = QColor("#A2627A")
    snapchat_yellow: QColor = QColor("#FFFC00")
    snow: QColor = QColor("#F5FEFD")
    snow_drift: QColor = QColor("#F8FBF8")
    soft_peach: QColor = QColor("#F6F1F4")
    solid_gray: QColor = QColor("#979797")
    solid_yellow: QColor = QColor("#FFE81C")
    sonic_gray: QColor = QColor("#757575")
    soot: QColor = QColor("#160D08")
    space_cadet: QColor = QColor("#1D2951")
    spanish_blue: QColor = QColor("#0070BB")
    spanish_gray: QColor = QColor("#989898")
    spanish_orange: QColor = QColor("#F06105")
    spanish_pink: QColor = QColor("#F7BFBE")
    spanish_red: QColor = QColor("#E60026")
    spanish_violet_purple: QColor = QColor("#4C2882")
    spanish_viridian: QColor = QColor("#007F5C")
    spanish_yellow: QColor = QColor("#F9D73D")
    spatial_white: QColor = QColor("#DEDDDB")
    spectrum_yellow: QColor = QColor("#FCE517")
    spice: QColor = QColor("#7A3803")
    spider: QColor = QColor("#040200")
    spring: QColor = QColor("#F3F0E8")
    spring_bud: QColor = QColor("#A7FC00")
    spring_orange: QColor = QColor("#FD963D")
    spruce: QColor = QColor("#2C3E4C")
    spruce_blue: QColor = QColor("#617178")
    squash: QColor = QColor("#C95B0C")
    squash_orange: QColor = QColor("#E89362")
    steel: QColor = QColor("#4682B4")
    steel_blue: QColor = QColor("#4682B4")
    steel_gray: QColor = QColor("#888B8D")
    steel_pink: QColor = QColor("#CC3366")
    stone: QColor = QColor("#59788E")
    stone_blue: QColor = QColor("#819EA8")
    stone_eagle: QColor = QColor("#D8D9DA")
    stone_gray: QColor = QColor("#928E85")
    stone_white: QColor = QColor("#FBF6F0")
    storm_gray: QColor = QColor("#B5BAB6")
    stormcloud: QColor = QColor("#4F666A")
    stout: QColor = QColor("#0F0B0A")
    strawberry: QColor = QColor("#E8888A")
    strong_magenta: QColor = QColor("#A1045A")
    strong_purple: QColor = QColor("#875692")
    studio: QColor = QColor("#7851A9")
    sugar_cookie: QColor = QColor("#F3EAAF")
    sulfur_yellow: QColor = QColor("#E8DE35")
    sun_yellow_ral: QColor = QColor("#EA9300")
    sunflower: QColor = QColor("#FFDA03")
    sunglow: QColor = QColor("#FFCC33")
    sunrise_orange: QColor = QColor("#E67451")
    sunset_orange: QColor = QColor("#FD5E53")
    sunset_purple: QColor = QColor("#A865B5")
    sunshine_yellow: QColor = QColor("#FFFD37")
    super_pink: QColor = QColor("#CF6BA9")
    swamp: QColor = QColor("#A8C090")
    swan_white: QColor = QColor("#FCFCFC")
    sweet_pink: QColor = QColor("#EE918D")
    syracuse_orange: QColor = QColor("#D44500")
    syrup: QColor = QColor("#481F01")
    tacao: QColor = QColor("#6F3096")
    taffy: QColor = QColor("#FA86C4")
    tall_ships_blue: QColor = QColor("#0E81B9")
    tan: QColor = QColor("#D2B48C")
    tangelo: QColor = QColor("#FC4C02")
    tangerine: QColor = QColor("#F78702")
    tangerine_yellow: QColor = QColor("#FFCC00")
    tango_red: QColor = QColor("#E4717A")
    tap_shoe: QColor = QColor("#2A2B2D")
    taupe: QColor = QColor("#483C32")
    taupe_gray: QColor = QColor("#8B8589")
    tawny: QColor = QColor("#80471C")
    tawny_orange: QColor = QColor("#B25A38")
    tawny_red: QColor = QColor("#CD5700")
    taxi_cab_yellow: QColor = QColor("#FDB813")
    tea_bag: QColor = QColor("#161311")
    tea_green: QColor = QColor("#D0F0C0")
    tea_rose: QColor = QColor("#F4C2C2")
    tea_rose_red: QColor = QColor("#F88379")
    teal: QColor = QColor("#008080")
    tech_black: QColor = QColor("#0D0E0E")
    templeton_gray: QColor = QColor("#788787")
    tender_yellow: QColor = QColor("#EDEEB7")
    tennis_court_green: QColor = QColor("#6C935C")
    terracotta: QColor = QColor("#E2725B")
    terracotta_red: QColor = QColor("#E2725B")
    thistle: QColor = QColor("#D8BFD8")
    thulian_pink: QColor = QColor("#DE6FA1")
    thunder: QColor = QColor("#BDB7AB")
    thyme: QColor = QColor("#5EDC1F")
    tickle_me_pink: QColor = QColor("#FC89AC")
    tiffany_blue: QColor = QColor("#81D8D0")
    tiger: QColor = QColor("#FC6A03")
    tiger_orange: QColor = QColor("#F96815")
    tigers_eye_orange: QColor = QColor("#E08D3C")
    timberwolf: QColor = QColor("#DBD7D2")
    titan_white: QColor = QColor("#EEEEFF")
    titanium_yellow: QColor = QColor("#EEE600")
    tomato: QColor = QColor("#FF6347")
    topaz_orange: QColor = QColor("#FFC87C")
    tornado_cloud: QColor = QColor("#121213")
    tortilla: QColor = QColor("#9A7B4F")
    tottenham_navy: QColor = QColor("#132257")
    tractor_red: QColor = QColor("#FD0E35")
    traditional_purple: QColor = QColor("#8E4585")
    traditional_royal_blue: QColor = QColor("#002366")
    traffic_purple: QColor = QColor("#913073")
    traffic_yellow_ral: QColor = QColor("#EFB700")
    transparent_yellow: QColor = QColor("#F4ECC2")
    tree_sap_orange: QColor = QColor("#CC7711")
    trombone: QColor = QColor("#D2B55B")
    tropical_rainforest: QColor = QColor("#00755E")
    trout: QColor = QColor("#979788")
    true_black: QColor = QColor("#0A0B0B")
    true_blue: QColor = QColor("#2D68C4")
    true_purple: QColor = QColor("#6A0DAD")
    true_silver: QColor = QColor("#8A8D8F")
    trust_yellow: QColor = QColor("#EEBE15")
    tufts_blue: QColor = QColor("#3E8EDE")
    tulip_pink: QColor = QColor("#FF8E8E")
    tumbleweed: QColor = QColor("#DEAA88")
    turkey_red: QColor = QColor("#A91101")
    turkish_blue: QColor = QColor("#4F97A3")
    turkish_rose: QColor = QColor("#B57281")
    turmeric: QColor = QColor("#F0CA28")
    turquoise: QColor = QColor("#40E0D0")
    turquoise_blue: QColor = QColor("#00FFEF")
    turquoise_green: QColor = QColor("#A0D6B4")
    tuscan_brown: QColor = QColor("#6F4E37")
    tuscan_red: QColor = QColor("#7C3030")
    tuscany: QColor = QColor("#FCD12A")
    twilight_lavender_purple: QColor = QColor("#8A496B")
    twitter_blue: QColor = QColor("#1DA1F2")
    tyrian_purple: QColor = QColor("#66023C")
    ucla_blue: QColor = QColor("#2774AE")
    ufo_green: QColor = QColor("#3CD070")
    ultimate_gray: QColor = QColor("#939597")
    ultra_pink: QColor = QColor("#FF6FFF")
    ultra_violet: QColor = QColor("#645394")
    ultramarine_blue: QColor = QColor("#120A8F")
    umber: QColor = QColor("#352315")
    university_of_central_arkansas_purple: QColor = QColor("#4F2D7F")
    university_of_tennessee_orange: QColor = QColor("#F77F00")
    unmellow_yellow: QColor = QColor("#FFFF66")
    unresolved_problem: QColor = QColor("#F3F2ED")
    up_maroon: QColor = QColor("#7B1113")
    upsdell_red: QColor = QColor("#AE2029")
    uranian_blue: QColor = QColor("#AFDBF5")
    urobilin: QColor = QColor("#E1AD21")
    us_air_force_academy_blue: QColor = QColor("#004F98")
    us_flag: QColor = QColor("#BF0A30")
    utah_crimson: QColor = QColor("#D3003F")
    valentine_pink: QColor = QColor("#E6A6BE")
    vampire_black: QColor = QColor("#0F0404")
    vampire_gray: QColor = QColor("#565051")
    vanilla: QColor = QColor("#F3E5AB")
    vanilla_ice: QColor = QColor("#F38FA9")
    venetian_red: QColor = QColor("#C80815")
    verdun: QColor = QColor("#487800")
    vermilion: QColor = QColor("#D74826")
    veronica: QColor = QColor("#A020F0")
    very_light_malachite_green: QColor = QColor("#64E986")
    very_pale_yellow: QColor = QColor("#FFFFBF")
    vibrant_yellow: QColor = QColor("#FFDA29")
    vintage_gray: QColor = QColor("#989692")
    vintage_orange: QColor = QColor("#C9995D")
    vintage_yellow: QColor = QColor("#CBA92B")
    violet: QColor = QColor("#8F00FF")
    violet_blue: QColor = QColor("#324AB2")
    virgin_lust_red: QColor = QColor("#E4181E")
    viridian: QColor = QColor("#609078")
    visa_blue: QColor = QColor("#1A1F71")
    vista_white: QColor = QColor("#FDFCFA")
    vivid_lime_green: QColor = QColor("#A4DE02")
    vivid_orange: QColor = QColor("#FF5E0E")
    vivid_reddish_purple: QColor = QColor("#870074")
    vivid_sky_blue: QColor = QColor("#00CCFF")
    vivid_tangelo: QColor = QColor("#EC7625")
    vivid_white: QColor = QColor("#F8F8F4")
    vivid_yellow: QColor = QColor("#FFE302")
    void: QColor = QColor("#010207")
    volt: QColor = QColor("#CEFF00")
    vulgar_purple_grape_jelly: QColor = QColor("#3E2F84")
    wageningen_green: QColor = QColor("#34B233")
    walnut: QColor = QColor("#432616")
    walnut_hull: QColor = QColor("#1B1813")
    warm_black: QColor = QColor("#004242")
    warm_gray: QColor = QColor("#6E6259")
    warm_yellow: QColor = QColor("#F4E98C")
    warriors_blue: QColor = QColor("#1D428A")
    watermelon: QColor = QColor("#FC6C85")
    web_orange: QColor = QColor("#FFA500")
    west_side_orange: QColor = QColor("#FF9916")
    wet_suit: QColor = QColor("#080706")
    wheat: QColor = QColor("#F5DEB3")
    wheat_orange: QColor = QColor("#F5DEB3")
    white: QColor = QColor("#FFFFFF")
    white_chocolate: QColor = QColor("#EDE6D6")
    white_diamond: QColor = QColor("#E7E8E2")
    white_dove: QColor = QColor("#F0EFE7")
    white_gray: QColor = QColor("#F2F3F5")
    white_heat: QColor = QColor("#FDF9EF")
    white_on_white: QColor = QColor("#EDEFEF")
    white_sand: QColor = QColor("#ECE9E4")
    white_smoke: QColor = QColor("#F5F5F5")
    white_whisper: QColor = QColor("#EAE8E1")
    white_wisp: QColor = QColor("#E9E9E4")
    whitewash: QColor = QColor("#FEFFFC")
    wild_orchid: QColor = QColor("#D470A2")
    wild_strawberry: QColor = QColor("#FF43A4")
    willpower_orange: QColor = QColor("#FD5602")
    wind_cave: QColor = QColor("#1F2024")
    window_gray: QColor = QColor("#989EA1")
    windsor_tan: QColor = QColor("#A75502")
    wine: QColor = QColor("#2C041C")
    wine_dregs_purple: QColor = QColor("#673147")
    wine_red: QColor = QColor("#722F37")
    winter_white: QColor = QColor("#DFDFD5")
    wood: QColor = QColor("#3F301D")
    wood_brown: QColor = QColor("#C19A6B")
    woodsmoke: QColor = QColor("#2B3230")
    x11_gray: QColor = QColor("#BBBCB6")
    xanadu: QColor = QColor("#738678")
    xanthic: QColor = QColor("#EEED09")
    xanthous_orange: QColor = QColor("#F1B42F")
    yacht_club_black: QColor = QColor("#222627")
    yale_blue: QColor = QColor("#00356B")
    yam: QColor = QColor("#CC5801")
    yam_orange: QColor = QColor("#E17E45")
    yankees_blue: QColor = QColor("#0C2340")
    yellow_cmyk: QColor = QColor("#FFFF00")
    yellow_cream: QColor = QColor("#EFDC75")
    yellow_jasmine: QColor = QColor("#CEB00D")
    yellow_orange: QColor = QColor("#D56F00")
    yellow_pantone: QColor = QColor("#FEDF00")
    yellow_rose: QColor = QColor("#FFF000")
    yellow_tan: QColor = QColor("#FFE36E")
    yin_mn_blue: QColor = QColor("#2E5090")
    yolk_yellow_pantone: QColor = QColor("#E2B051")
    young_night: QColor = QColor("#232323")
    your: QColor = QColor("#FFC0C0")
    zinc_yellow_ral: QColor = QColor("#F2CB2E")
    zinnwaldite_brown: QColor = QColor("#2C1608")
    zombie_gray: QColor = QColor("#63645E")

ColorTypeVar: TypeAlias = Union[QColor, QRgba64, Any, 
                                Qt.GlobalColor, Colors, 
                                str, int]